@@ -3,22 +3,21 @@
 import json
 import uuid
 
-from le_utils.constants import content_kinds, exercises, file_formats, format_presets, languages
+from le_utils.constants import content_kinds, exercises, file_formats, format_presets, languages, roles
 
-from ricecooker.classes.files import NodeFile
 from .licenses import License
 from .. import config, __version__
 from ..exceptions import InvalidNodeException
 
 MASTERY_MODELS = [id for id, name in exercises.MASTERY_MODELS]
-
+ROLES = [id for id, name in roles.choices]
 
 class Node(object):
     """ Node: model to represent all nodes in the tree """
     license = None
     language = None
 
-    def __init__(self, title, language=None, description=None, thumbnail=None, files=None):
+    def __init__(self, title, language=None, description=None, thumbnail=None, files=None, derive_thumbnail=False):
         self.files = []
         self.children = []
         self.descendants = []
@@ -26,9 +25,9 @@ class Node(object):
         self.node_id = None
         self.content_id = None
         self.title = title
-        self.hashed_file_name = None
         self.set_language(language)
         self.description = description or ""
+        self.derive_thumbnail = derive_thumbnail
 
         for f in files or []:
             self.add_file(f)
@@ -90,12 +89,18 @@ class Node(object):
         if file_to_add not in self.files:
             self.files.append(file_to_add)
 
-    def derive_thumbnail(self):
-        pass
+    def generate_thumbnail(self):
+        """Each node subclass implements its own thumbnail generation logic.
+
+        Returns:
+            A Thumbnail object (unprocessed) or None.
+        """
+        return None
 
     def has_thumbnail(self):
         from .files import ThumbnailFile
         return any(f for f in self.files if isinstance(f, ThumbnailFile))
+        # TODO deep check: f.process_file() and check f.filename is not None
 
     def set_thumbnail(self, thumbnail):
         """ set_thumbnail: Set node's thumbnail
@@ -112,7 +117,8 @@ class Node(object):
 
     def get_thumbnail_preset(self):
         """
-        Returns the format preset corresponding to this Node's type, or None if the node doesn't have a format preset.
+        Returns the format preset corresponding to this Node's type,
+        or None if the node doesn't have a format preset.
         """
         if isinstance(self, ChannelNode):
             return format_presets.CHANNEL_THUMBNAIL
@@ -132,23 +138,30 @@ class Node(object):
             return None
 
     def process_files(self):
+        """Processes all the files associated with this Node, including:
+        - download files if not present in the local storage
+        - convert and compress video files
+        - (optionally) generate thumbnail file from the node's content
+        Returns: content-hash based filenames of all the files for this node
         """
-        Processes all the files associated with this Node. Files are downloaded if not present in the local storage.
-        Creates and processes a NodeFile containing this Node's metadata.
-        :return: A list of names of all the processed files.
-        """
-        file_names = []
-        for f in self.files:
-            file_names.append(f.process_file())
+        filenames = []
+        for file in self.files:
+            filenames.append(file.process_file())
 
-        if not self.has_thumbnail() and config.THUMBNAILS:
-            file_names.append(self.derive_thumbnail())
+        # Auto-generation of thumbnails happens here if derive_thumbnail or config.THUMBNAILS is set
+        if not self.has_thumbnail() and (config.THUMBNAILS or self.derive_thumbnail):
+            thumbnail_file = self.generate_thumbnail()
+            if thumbnail_file:
+                thumbnail_filename = thumbnail_file.process_file()
+                if thumbnail_filename:
+                    self.set_thumbnail(thumbnail_file)
+                    filenames.append(thumbnail_filename)
+                else:
+                    pass  # failed to generate thumbnail
+            else:
+                pass  # method generate_thumbnail is not implemented or no suitable source file found
 
-        node_file = NodeFile(self.to_dict())
-        self.hashed_file_name = node_file.process_file()
-        file_names.append(self.hashed_file_name)
-
-        return file_names
+        return filenames
 
     def count(self):
         """ count: get number of nodes in tree
@@ -160,6 +173,18 @@ class Node(object):
             total += child.count()
         return total
 
+    def get_topic_count(self):
+        """ get_topic_count: get number of topics in tree
+            Args: None
+            Returns: int
+        """
+        total = 0
+        if self.kind == content_kinds.TOPIC or self.kind == "Channel":
+            total = 1
+            for child in self.children:
+                total += child.get_topic_count()
+        return total
+
     def get_non_topic_descendants(self):
         if len(self.descendants) == 0:
             for child_node in self.children:
@@ -178,14 +203,15 @@ class Node(object):
         for child in self.children:
             child.print_tree(indent + 1)
 
-    def test_tree(self):
-        """ test_tree: validate all nodes in this tree
-            Args: None
-            Returns: boolean indicating if tree is valid
+    def validate_tree(self):
+        """
+        Validate all nodes in this tree recusively.
+          Args: None
+          Returns: boolean indicating if tree is valid
         """
         self.validate()
         for child in self.children:
-            assert child.test_tree()
+            assert child.validate_tree()
         return True
 
     def validate(self):
@@ -195,13 +221,18 @@ class Node(object):
         """
         from .files import File
 
-        assert self.source_id is not None, "Assumption Failed: Node must have an id"
+        assert self.source_id is not None, "Assumption Failed: Node must have a source_id"
         assert isinstance(self.title, str), "Assumption Failed: Node title is not a string"
+        assert len(self.title.strip()) > 0, "Assumption Failed: Node title cannot be empty"
         assert isinstance(self.description, str) or self.description is None, "Assumption Failed: Node description is not a string"
         assert isinstance(self.children, list), "Assumption Failed: Node children is not a list"
         for f in self.files:
             assert isinstance(f, File), "Assumption Failed: files must be file class"
             f.validate()
+
+        source_ids = [c.source_id for c in self.children]
+        duplicates = set([x for x in source_ids if source_ids.count(x) > 1])
+        assert len(duplicates) == 0, "Assumption Failed: Node must have unique source id among siblings ({} appears multiple times)".format(duplicates)
         return True
 
 
@@ -242,7 +273,7 @@ class ChannelNode(Node):
         super(ChannelNode, self).truncate_fields()
 
     def to_dict(self):
-        """ to_dict: puts data in format CC expects
+        """ to_dict: puts channel data into the format that Kolibri Studio expects
             Args: None
             Returns: dict of channel data
         """
@@ -282,16 +313,22 @@ class TreeNode(Node):
             license (str or <License>): content's license
             description (str): description of content (optional)
             author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
             thumbnail (str): local path or url to thumbnail image (optional)
             files ([<File>]): list of file objects for node (optional)
+            tags ([str]): list of tags for node (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
     """
-    def __init__(self, source_id, title, author="", extra_fields=None, domain_ns=None, **kwargs):
+    def __init__(self, source_id, title, author="", aggregator="", provider="", tags=None, extra_fields=None, domain_ns=None, **kwargs):
         # Map parameters to model variables
         assert isinstance(source_id, str), "source_id must be a string"
         self.source_id = source_id
         self.author = author or ""
+        self.aggregator = aggregator or ""
+        self.provider = provider or ""
+        self.tags = tags or []
         self.domain_ns = domain_ns
         self.questions = self.questions if hasattr(self, 'questions') else [] # Needed for to_dict method
         self.extra_fields = extra_fields or {}
@@ -320,13 +357,21 @@ class TreeNode(Node):
             config.print_truncate("author", self.source_id, self.author, kind=self.kind)
             self.author = self.author[:config.MAX_AUTHOR_LENGTH]
 
+        if self.aggregator and len(self.aggregator) > config.MAX_AGGREGATOR_LENGTH:
+            config.print_truncate("aggregator", self.source_id, self.aggregator, kind=self.kind)
+            self.aggregator = self.aggregator[:config.MAX_AGGREGATOR_LENGTH]
+
+        if self.provider and len(self.provider) > config.MAX_PROVIDER_LENGTH:
+            config.print_truncate("provider", self.source_id, self.provider, kind=self.kind)
+            self.provider = self.provider[:config.MAX_PROVIDER_LENGTH]
+
         self.license and self.license.truncate_fields()
 
         super(TreeNode, self).truncate_fields()
 
 
     def to_dict(self):
-        """ to_dict: puts data in format CC expects
+        """ to_dict: puts Topic or Content node data into the format that Kolibri Studio expects
             Args: None
             Returns: dict of channel data
         """
@@ -339,7 +384,10 @@ class TreeNode(Node):
             "source_domain": self.domain_ns.hex,
             "source_id": self.source_id,
             "author": self.author,
+            "aggregator": self.aggregator,
+            "provider": self.provider,
             "files" : [f.to_dict() for f in self.files if f and f.filename], # Filter out failed downloads
+            "tags": self.tags,
             "kind": self.kind,
             "license": None,
             "license_description": None,
@@ -354,9 +402,15 @@ class TreeNode(Node):
             Returns: boolean indicating if content node is valid
         """
         assert isinstance(self.author, str) , "Assumption Failed: Author is not a string"
+        assert isinstance(self.aggregator, str) , "Assumption Failed: Aggregator is not a string"
+        assert isinstance(self.provider, str) , "Assumption Failed: Provider is not a string"
         assert isinstance(self.files, list), "Assumption Failed: Files is not a list"
         assert isinstance(self.questions, list), "Assumption Failed: Questions is not a list"
         assert isinstance(self.extra_fields, dict), "Assumption Failed: Extra fields is not a dict"
+        assert isinstance(self.tags, list), "Assumption Failed: Tags is not a list"
+        for tag in self.tags:
+            assert isinstance(tag, str) , "Assumption Failed: Tag is not a string"
+            assert len(tag) <= 50, "ERROR: tag " + tag + " is too long. Tags should be 50 chars or less."
         return super(TreeNode, self).validate()
 
 
@@ -370,12 +424,16 @@ class TopicNode(TreeNode):
             title (str): content's title
             description (str): description of content (optional)
             thumbnail (str): local path or url to thumbnail image (optional)
+            derive_thumbnail (bool): set to generate tiled thumbnail from children (optional)
     """
     kind = content_kinds.TOPIC
-    def derive_thumbnail(self):
+
+    def generate_thumbnail(self):
+        """Generate a ``TiledThumbnailFile`` based on the descendants.
+        Returns: a Thumbnail file or None.
+        """
         from .files import TiledThumbnailFile
-        self.set_thumbnail(TiledThumbnailFile(self.get_non_topic_descendants()))
-        return self.thumbnail.get_filename()
+        return TiledThumbnailFile(self.get_non_topic_descendants())
 
     def validate(self):
         """ validate: Makes sure topic is valid
@@ -400,14 +458,19 @@ class ContentNode(TreeNode):
             license (str or <License>): content's license
             description (str): description of content (optional)
             author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
+            role (str): set to roles.COACH for teacher-facing materials (default roles.LEARNER)
             thumbnail (str): local path or url to thumbnail image (optional)
+            derive_thumbnail (bool): set to generate thumbnail from content (optional)
             files ([<File>]): list of file objects for node (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
     """
     required_file_format = None
 
-    def __init__(self, source_id, title, license, license_description=None, copyright_holder=None, **kwargs):
+    def __init__(self, source_id, title, license, role=roles.LEARNER, license_description=None, copyright_holder=None, **kwargs):
+        self.role = role
         self.set_license(license, copyright_holder=copyright_holder, description=license_description)
         super(ContentNode, self).__init__(source_id, title, **kwargs)
 
@@ -427,7 +490,9 @@ class ContentNode(TreeNode):
             Args: None
             Returns: boolean indicating if content node is valid
         """
+        assert self.role in ROLES, "Assumption Failed: Role must be one of the following {}".format(ROLES)
         assert isinstance(self.license, str) or isinstance(self.license, License), "Assumption Failed: License is not a string or license object"
+        self.license.validate()
         # if self.required_file_format:
         #     files_valid = False
         #     #not any(f for f in self.files if isinstance(f, DownloadFile))
@@ -450,13 +515,17 @@ class ContentNode(TreeNode):
             "source_domain": self.domain_ns.hex,
             "source_id": self.source_id,
             "author": self.author,
+            "aggregator": self.aggregator,
+            "provider": self.provider,
             "files" : [f.to_dict() for f in filter(lambda x: x and x.filename, self.files)], # Filter out failed downloads
+            "tags": self.tags,
             "kind": self.kind,
             "license": self.license.license_id,
             "license_description": self.license.description,
             "copyright_holder": self.license.copyright_holder,
             "questions": [question.to_dict() for question in self.questions],
             "extra_fields": json.dumps(self.extra_fields),
+            "role": self.role,
         }
 
 
@@ -470,8 +539,10 @@ class VideoNode(ContentNode):
             title (str): content's title
             license (str or <License>): content's license
             author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
             description (str): description of content (optional)
-            derive_thumbnail (bool): indicates whether to derive thumbnail from video (optional)
+            derive_thumbnail (bool): set to generate thumbnail from video (optional)
             thumbnail (str): local path or url to thumbnail image (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
@@ -480,48 +551,50 @@ class VideoNode(ContentNode):
     kind = content_kinds.VIDEO
     required_file_format = file_formats.MP4
 
-    def __init__(self, source_id, title, license, derive_thumbnail=False, **kwargs):
-        self.generate_thumbnail = derive_thumbnail
+    def __init__(self, source_id, title, license, **kwargs):
         super(VideoNode, self).__init__(source_id, title, license, **kwargs)
 
-    def process_files(self):
-        """ download_files: Download video's files
-            Args: None
-            Returns: None
-        """
-        from .files import VideoFile, ExtractedVideoThumbnailFile, WebVideoFile
-
-        downloaded = super(VideoNode, self).process_files()
-
-        try:
-            # Extract thumbnail if one hasn't been provided and derive_thumbnail is set
-            if self.generate_thumbnail and not self.has_thumbnail():
-                videos = [f for f in self.files if isinstance(f, VideoFile) or isinstance(f, WebVideoFile)]
-                assert len(videos) > 0 and videos[0].filename, "Cannot extract thumbnail (No videos found on node {0})".format(self.source_id)
-
-                self.set_thumbnail(ExtractedVideoThumbnailFile(config.get_storage_path(videos[0].filename)))
-                downloaded.append(self.thumbnail.get_filename())
-
-        except AssertionError as ae:
-            config.LOGGER.warning(ae)
-
-        return downloaded
+    def generate_thumbnail(self):
+        from .files import VideoFile, WebVideoFile, ExtractedVideoThumbnailFile
+        video_files = [f for f in self.files if isinstance(f, VideoFile) or isinstance(f, WebVideoFile)]
+        if video_files:
+            video_file = video_files[0]
+            if video_file.filename and not video_file.error:
+                storage_path = config.get_storage_path(video_file.filename)
+                return ExtractedVideoThumbnailFile(storage_path)
+        return None
 
     def validate(self):
         """ validate: Makes sure video is valid
             Args: None
             Returns: boolean indicating if video is valid
         """
-        from .files import VideoFile, WebVideoFile
+        from .files import VideoFile, WebVideoFile, SubtitleFile, YouTubeSubtitleFile
         try:
             assert self.kind == content_kinds.VIDEO, "Assumption Failed: Node should be a video"
             assert self.questions == [], "Assumption Failed: Video should not have questions"
             assert len(self.files) > 0, "Assumption Failed: Video must have at least one video file"
 
             # Check if there are any .mp4 files if there are video files (other video types don't have paths)
-            assert any(f for f in self.files if isinstance(f, VideoFile) or isinstance(f, WebVideoFile)), "Assumption Failed: Video should have at least one .mp4 file"
+            assert any(f for f in self.files if isinstance(f, VideoFile) or isinstance(f, WebVideoFile)), "Assumption Failed: Video node should have at least one video file"
+
+            # Ensure that there is only one subtitle file per language code
+            new_files = []
+            language_codes_seen = set()
+            for file in self.files:
+                if isinstance(file, SubtitleFile) or isinstance(file, YouTubeSubtitleFile):
+                    language_code = file.language
+                    if language_code not in language_codes_seen:
+                        new_files.append(file)
+                        language_codes_seen.add(language_code)
+                    else:
+                        config.LOGGER.warning('Skipping duplicate subs for ' + language_code + ' from path ' + file.path)
+                else:
+                    new_files.append(file)
+            self.files = new_files
 
             return super(VideoNode, self).validate()
+
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
 
@@ -536,8 +609,11 @@ class AudioNode(ContentNode):
             title (str): content's title
             license (str or <License>): content's license
             author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
             description (str): description of content (optional)
             thumbnail (str): local path or url to thumbnail image (optional)
+            derive_thumbnail (bool): set to generate waveform thumbnail (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
             files ([<File>]): list of file objects for node (optional)
@@ -545,6 +621,17 @@ class AudioNode(ContentNode):
     kind = content_kinds.AUDIO
     required_file_format = file_formats.MP3
 
+    def generate_thumbnail(self):
+        from .files import AudioFile, ExtractedAudioThumbnailFile
+        audio_files = [f for f in self.files if isinstance(f, AudioFile)]
+        if audio_files:
+            audio_file = audio_files[0]
+            if audio_file.filename and not audio_file.error:
+                storage_path = config.get_storage_path(audio_file.filename)
+                return ExtractedAudioThumbnailFile(storage_path)
+        return None
+
+
     def validate(self):
         """ validate: Makes sure audio is valid
             Args: None
@@ -555,7 +642,7 @@ class AudioNode(ContentNode):
             assert self.kind == content_kinds.AUDIO, "Assumption Failed: Node should be audio"
             assert self.questions == [], "Assumption Failed: Audio should not have questions"
             assert len(self.files) > 0, "Assumption Failed: Audio should have at least one file"
-            assert any(filter(lambda f: isinstance(f, AudioFile), self.files)), "Assumption Failed: Audio should have at least one audio file"
+            assert [f for f in self.files if isinstance(f, AudioFile)], "Assumption Failed: Audio should have at least one audio file"
             return super(AudioNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
@@ -564,37 +651,59 @@ class AudioNode(ContentNode):
 class DocumentNode(ContentNode):
     """ Model representing documents in channel
 
-        Documents must be pdf format
+        Documents must be in PDF or ePub format
 
         Attributes:
             source_id (str): content's original id
             title (str): content's title
             license (str or <License>): content's license
             author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
             description (str): description of content (optional)
             thumbnail (str): local path or url to thumbnail image (optional)
+            derive_thumbnail (bool): automatically generate thumbnail (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
             files ([<File>]): list of file objects for node (optional)
     """
     kind = content_kinds.DOCUMENT
-    required_file_format = file_formats.PDF
+    required_file_format = file_formats.PDF # TODO(ivan) change ro allowed_formats
 
     def validate(self):
-        """ validate: Makes sure document is valid
+        """ validate: Makes sure document node contains at least one EPUB or PDF
             Args: None
             Returns: boolean indicating if document is valid
         """
-        from .files import DocumentFile
+        from .files import DocumentFile, EPubFile
         try:
             assert self.kind == content_kinds.DOCUMENT, "Assumption Failed: Node should be a document"
             assert self.questions == [], "Assumption Failed: Document should not have questions"
             assert len(self.files) > 0, "Assumption Failed: Document should have at least one file"
-            assert any(filter(lambda f: isinstance(f, DocumentFile), self.files)), "Assumption Failed: Document should have at least one document file"
+            assert [f for f in self.files if isinstance(f, DocumentFile) or isinstance(f, EPubFile)], \
+                "Assumption Failed: Document should have at least one document file"
             return super(DocumentNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
 
+    def generate_thumbnail(self):
+        from .files import DocumentFile, EPubFile, ExtractedPdfThumbnailFile, ExtractedEPubThumbnailFile
+        pdf_files = [f for f in self.files if isinstance(f, DocumentFile)]
+        epub_files = [f for f in self.files if isinstance(f, EPubFile)]
+        if pdf_files and epub_files:
+            raise InvalidNodeException("Invalid node (both PDF and ePub provided): {} - {}".format(self.title, self.__dict__))
+        elif pdf_files:
+            pdf_file = pdf_files[0]
+            if pdf_file.filename and not pdf_file.error:
+                storage_path = config.get_storage_path(pdf_file.filename)
+                return ExtractedPdfThumbnailFile(storage_path)
+        elif epub_files:
+            epub_file = epub_files[0]
+            if epub_file.filename and not epub_file.error:
+                storage_path = config.get_storage_path(epub_file.filename)
+                return ExtractedEPubThumbnailFile(storage_path)
+        return None
+
 
 class HTML5AppNode(ContentNode):
     """ Model representing a zipped HTML5 application
@@ -607,8 +716,11 @@ class HTML5AppNode(ContentNode):
             title (str): content's title
             license (str or <License>): content's license
             author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
             description (str): description of content (optional)
             thumbnail (str): local path or url to thumbnail image (optional)
+            derive_thumbnail (bool): generate thumbnail from largest image inside zip (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
             files ([<File>]): list of file objects for node (optional)
@@ -616,6 +728,17 @@ class HTML5AppNode(ContentNode):
     kind = content_kinds.HTML5
     required_file_format = file_formats.HTML5
 
+    def generate_thumbnail(self):
+        from .files import HTMLZipFile, ExtractedHTMLZipThumbnailFile
+        html5_files = [f for f in self.files if isinstance(f, HTMLZipFile)]
+        if html5_files:
+            html_file = html5_files[0]
+            if html_file.filename and not html_file.error:
+                storage_path = config.get_storage_path(html_file.filename)
+                return ExtractedHTMLZipThumbnailFile(storage_path)
+        else:
+            return None
+
     def validate(self):
         """ validate: Makes sure HTML5 app is valid
             Args: None
@@ -625,9 +748,44 @@ class HTML5AppNode(ContentNode):
         try:
             assert self.kind == content_kinds.HTML5, "Assumption Failed: Node should be an HTML5 app"
             assert self.questions == [], "Assumption Failed: HTML should not have questions"
-            assert any(filter(lambda f: isinstance(f, HTMLZipFile), self.files)), "Assumption Failed: HTML should have at least one html file"
+            assert [f for f in self.files if isinstance(f, HTMLZipFile)], "Assumption Failed: HTML should have at least one html file"
             return super(HTML5AppNode, self).validate()
+        except AssertionError as ae:
+            raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
 
+
+class H5PAppNode(ContentNode):
+    """ Model representing a H5P content nodes
+
+        The .h5p file is self-contained and inlcuding media and javascript libs.
+
+        Attributes:
+            source_id (str): content's original id
+            title (str): content's title
+            license (str or <License>): content's license
+            author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
+            description (str): description of content (optional)
+            thumbnail (str): local path or url to thumbnail image (optional)
+            extra_fields (dict): any additional data needed for node (optional)
+            domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
+            files ([<File>]): list of file objects for node (optional)
+    """
+    kind = content_kinds.H5P
+    required_file_format = file_formats.H5P
+
+    def validate(self):
+        """ validate: Makes sure H5P app is valid
+            Args: None
+            Returns: boolean indicating if H5P app is valid
+        """
+        from .files import H5PFile
+        try:
+            assert self.kind == content_kinds.H5P, "Assumption Failed: Node should be an H5P app"
+            assert self.questions == [], "Assumption Failed: HTML should not have questions"
+            assert [f for f in self.files if isinstance(f, H5PFile)], "Assumption Failed: H5PAppNode should have at least one h5p file"
+            return super(H5PAppNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
 
@@ -643,6 +801,8 @@ class ExerciseNode(ContentNode):
             title (str): content's title
             license (str or <License>): content's license
             author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
             description (str): description of content (optional)
             exercise_data ({mastery_model:str, randomize:bool, m:int, n:int}): data on mastery requirements (optional)
             thumbnail (str): local path or url to thumbnail image (optional)
@@ -662,8 +822,8 @@ class ExerciseNode(ContentNode):
             exercise_data = {"mastery_model": exercise_data}
 
         exercise_data.update({
-            'mastery_model': exercise_data.get('mastery_model') or exercises.M_OF_N,
-            'randomize': exercise_data.get('randomize') or True,
+            'mastery_model': exercise_data.get('mastery_model', exercises.M_OF_N),
+            'randomize': exercise_data.get('randomize', True),
         })
 
         super(ExerciseNode, self).__init__(source_id, title, license, extra_fields=exercise_data, **kwargs)
@@ -680,9 +840,8 @@ class ExerciseNode(ContentNode):
         self.questions += [question]
 
     def process_files(self):
-        """ process_files: goes through question fields and replaces image strings
-            Args: None
-            Returns: None
+        """Goes through question fields and replaces image strings
+        Returns: content-hash based filenames of all the required image files
         """
         config.LOGGER.info("\t*** Processing images for exercise: {}".format(self.title))
         downloaded = super(ExerciseNode, self).process_files()
@@ -731,8 +890,9 @@ class ExerciseNode(ContentNode):
 
             # Check if questions are correct
             assert any(self.questions), "Assumption Failed: Exercise does not have a question"
-            assert all(filter(lambda q: q.validate(), self.questions)), "Assumption Failed: Exercise has invalid question"
-            assert self.extra_fields['mastery_model'] in MASTERY_MODELS, "Assumption Failed: Unrecognized mastery model {}".format(self.extra_fields['mastery_model'])
+            assert all([q.validate() for q in self.questions]), "Assumption Failed: Exercise has invalid question"
+            assert self.extra_fields['mastery_model'] in MASTERY_MODELS, \
+                "Assumption Failed: Unrecognized mastery model {}".format(self.extra_fields['mastery_model'])
             return super(ExerciseNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
@@ -742,3 +902,88 @@ class ExerciseNode(ContentNode):
             q.truncate_fields()
 
         super(ExerciseNode, self).truncate_fields()
+
+class SlideshowNode(ContentNode):
+    """ Model representing Slideshows
+
+        Slideshows are sequences of "Slides", a combination of an image and caption.
+        Slides are shown in a specified sequential order.
+
+        Attributes:
+            source_id (str): content's original id
+            title (str): content's title
+            license (str or <License>): content's license
+            author (str): who created the content (optional)
+            aggregator (str): website or org hosting the content collection but not necessarily the creator or copyright holder (optional)
+            provider (str): organization that commissioned or is distributing the content (optional)
+            description (str): description of content (optional)
+            files ([<SlideImageFile>]): images associated with slides
+            thumbnail (str): local path or url to thumbnail image (optional)
+            extra_fields (dict): any additional data needed for node (optional)
+            domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
+    """
+    kind = content_kinds.SLIDESHOW
+
+    def __init__(self, source_id, title, license, slideshow_data=None, **kwargs):
+        if slideshow_data:
+            extra_fields = {'slideshow_data': slideshow_data}
+        else:
+            extra_fields = {'slideshow_data':[]}
+        # THe Node base class' __init__ method has:
+        #       for f in files or []:
+        #           self.add_file(f)
+        super(SlideshowNode, self).__init__(source_id, title, license, extra_fields=extra_fields, **kwargs)
+
+    def add_file(self, file_to_add):
+        """
+        Add a the SlideImageFile to the node's files and append slideshow metadata
+        to extra_fields['slideshow_data'] (list).
+        Args: file (SlideshowNode or ThumbnailFile): file model to add to node
+        Returns: None
+        """
+        from .files import ThumbnailFile, SlideImageFile
+        assert isinstance(file_to_add, ThumbnailFile) or isinstance(file_to_add, SlideImageFile), "Files being added must be instances of a subclass of File class"
+
+        if file_to_add not in self.files:
+            filename = file_to_add.get_filename()
+            if filename:
+                checksum, ext = filename.split('.')  # <md5sum(contents)>.[png|jpg|jpeg]
+            else:
+                raise ValueError('filename not available')
+
+            #
+            # Appending to extra_fields is only necessary for SlideImageFile instances
+            if isinstance(file_to_add, SlideImageFile):
+                #
+                # Find the idx of sort_order.next()
+                slideshow_image_files = [f for f in self.files if isinstance(f,SlideImageFile)]
+                idx = len(slideshow_image_files)  # next available index, assuming added in desired order
+                #
+                # Add slideshow data to extra_fields['slideshow_data'] (aka manifest)
+                slideshow_data = self.extra_fields['slideshow_data']
+                slideshow_data.append(
+                    {
+                        'caption': file_to_add.caption,
+                        'descriptive_text': file_to_add.descriptive_text,
+                        'sort_order': idx,
+                        'checksum': checksum,
+                        'extension': ext
+                    }
+                )
+                self.extra_fields['slideshow_data'] = slideshow_data
+
+            #
+            # Add node->file link
+            file_to_add.node = self
+            self.files.append(file_to_add)
+
+    def validate(self):
+        from .files import SlideImageFile, ThumbnailFile
+        try:
+            assert [f for f in self.files if isinstance(f, SlideImageFile)], \
+                "Assumption Failed: SlideshowNode must have at least one SlideImageFile file."
+            assert all([isinstance(f, SlideImageFile) or isinstance(f, ThumbnailFile) for f in self.files]), \
+                   "Assumption Failed: SlideshowNode files must be of type SlideImageFile or ThumbnailFile."
+        except AssertionError as ae:
+            raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
+        super(SlideshowNode, self).validate()
