@@ -14,11 +14,10 @@ from ..exceptions import UnknownQuestionTypeError, InvalidQuestionException
 from .files import _ExerciseImageFile, _ExerciseGraphieFile, _ExerciseBase64ImageFile
 from pressurecooker.encodings import get_base64_encoding
 
-WEB_GRAPHIE_URL_REGEX = r'web\+graphie:([^\)]+)'
-FILE_REGEX = r'!\[([^\]]+)?\]\(([^\)]+)\)'
-IMG_REGEX = r'<\s*img[^>]+>'
-IMG_SRC_REGEX = r'\ssrc\s*=\"([^"]+)\"'
-IMG_ALT_REGEX = r'\salt\s*=\"([^"]+)\"'
+
+WEB_GRAPHIE_URL_REGEX = r'web\+graphie:(?P<rawpath>[^\)]+)'  # match web_graphie:{{path}}
+MARKDOWN_IMAGE_REGEX = r'!\[([^\]]+)?\]\(([^\)]+?)\)'        # match ![{{smth}}]({{url}})
+
 
 class BaseQuestion:
     """ Base model representing exercise questions
@@ -36,11 +35,11 @@ class BaseQuestion:
     def __init__(self, id, question, question_type, answers=None, hints=None, raw_data="", source_url=None, randomize=False):
         self.question = question
         self.question_type = question_type
-        self.files=[]
+        self.files = []
         self.answers = answers if answers is not None else []
         self.hints = [] if hints is None else [hints] if isinstance(hints,str) else hints
         self.raw_data = raw_data
-        self.source_id=id
+        self.source_id = id
         self.source_url = source_url
         self.randomize = randomize
         self.id = uuid.uuid5(uuid.NAMESPACE_DNS, id)
@@ -74,7 +73,7 @@ class BaseQuestion:
                 correct (bool): indicates if answer is correct
             Returns: dict of formatted answer
         """
-        return {"answer": str(answer), "correct":correct}
+        return {"answer": str(answer), "correct": correct}
 
     def process_question(self):
         """ process_question: Parse data that needs to have image strings processed
@@ -109,7 +108,7 @@ class BaseQuestion:
         self.files += question_files + answer_files + hint_files
         return [f.filename for f in self.files]
 
-    def set_images(self, text):
+    def set_images(self, text, parse_html=True):
         """ set_images: Replace image strings with downloaded image checksums
             Args:
                 text (str): text to parse for image strings
@@ -118,8 +117,11 @@ class BaseQuestion:
         """
         # Set up return values and regex
         file_list = []
-        processed_string = self.parse_html(text)
-        reg = re.compile(FILE_REGEX, flags=re.IGNORECASE)
+        if parse_html:
+            processed_string = self.parse_html(text)
+        else:
+            processed_string = text
+        reg = re.compile(MARKDOWN_IMAGE_REGEX, flags=re.IGNORECASE)
         matches = reg.findall(processed_string)
 
         # Parse all matches
@@ -138,7 +140,7 @@ class BaseQuestion:
             Returns: string with properly formatted images
         """
         bs = BeautifulSoup(text, "html5lib")
-        file_reg = re.compile(FILE_REGEX, flags=re.IGNORECASE)
+        file_reg = re.compile(MARKDOWN_IMAGE_REGEX, flags=re.IGNORECASE)
         tags = bs.findAll('img')
 
         for tag in tags:
@@ -152,37 +154,43 @@ class BaseQuestion:
         return html.unescape(bs.find('body').renderContents().decode('utf-8'))
 
     def set_image(self, text):
-        """ set_image: Replace image string with downloaded image checksum
-            Args:
-                text (str): text to parse for image strings
-            Returns:string with checksums in place of image strings and
-                list of files that were downloaded from string
         """
-        # Make sure image hasn't already been replaced
+        Save image resource at `text` (path or url) to storage, then return the
+        replacement string and the necessary exercicse image file object.
+        Args:
+          - text (str): path or url to parse as an exercise image resource
+        Returns: (new_text, files)
+          - `new_text` (str): replacement string for the original `text` string
+          - `files` (list): list of files that were downloaded from `text`
+        """
+        # Make sure `text` hasn't already been processed
         if exercises.CONTENT_STORAGE_PLACEHOLDER in text:
             return text, []
-
-        # Set up return values and regex
-        exercise_file = None
-        path_text = text.strip().replace('\\n', '')
-        graphie_reg = re.compile(WEB_GRAPHIE_URL_REGEX, flags=re.IGNORECASE)
-        graphie_match = graphie_reg.match(path_text)
-        # If it is a web+graphie, download svg and json files,
-        # Otherwise, download like other files
+        # Strip `text` of whitespace
+        stripped_text = text.strip().replace('\\n', '')
+        # If `stripped_text` is a web+graphie: path, we need special processing
+        graphie_regex = re.compile(WEB_GRAPHIE_URL_REGEX, flags=re.IGNORECASE)
+        graphie_match = graphie_regex.match(stripped_text)
         if graphie_match:
-            path_text = graphie_match.group().replace("web+graphie:", "")
-            exercise_file = _ExerciseGraphieFile(path_text)
-        elif get_base64_encoding(text):
-            exercise_file = _ExerciseBase64ImageFile(path_text)
+            is_web_plus_graphie = True
+            graphie_rawpath = graphie_match.groupdict()['rawpath']
+            graphie_path = graphie_rawpath.replace("//", "https://")
+            exercise_image_file = _ExerciseGraphieFile(graphie_path)
+        elif get_base64_encoding(stripped_text):
+            is_web_plus_graphie = False
+            exercise_image_file = _ExerciseBase64ImageFile(stripped_text)
         else:
-            exercise_file = _ExerciseImageFile(path_text)
-
-        exercise_file.assessment_item = self
-        filename = exercise_file.process_file()
-
-        text = text.replace(path_text, exercises.CONTENT_STORAGE_FORMAT.format(exercise_file.get_replacement_str()))
-
-        return text, [exercise_file]
+            is_web_plus_graphie = False
+            exercise_image_file = _ExerciseImageFile(stripped_text)
+        # Setup link to assessment item
+        exercise_image_file.assessment_item = self
+        # Process file to make the replacement_str available
+        _filename = exercise_image_file.process_file()
+        # Get `new_text` = the replacement path for the image resource
+        new_text = exercises.CONTENT_STORAGE_FORMAT.format(exercise_image_file.get_replacement_str())
+        if is_web_plus_graphie:     # need to put back the `web+graphie:` prefix
+            new_text = "web+graphie:" + new_text
+        return new_text, [exercise_image_file]
 
     def validate(self):
         """ validate: Makes sure question is valid
@@ -197,21 +205,21 @@ class BaseQuestion:
         for a in self.answers:
             assert isinstance(a, dict), "Assumption Failed: Answer in answer list is not a dict"
         for h in self.hints:
-            assert isinstance(h, str), "Assumption Failed: Hint in hint list is not a string"
+            assert isinstance(h, str), "Assumption Failed: Hint in hints list is not a string"
         return True
 
 
 class PerseusQuestion(BaseQuestion):
-    """ Model representing perseus questions
+    """
+    Model representing existing perseus questions. These questions are already
+    formatted in the `.perseus` format and can therefore be created by passing in
+    the `raw_data` (str) attribute, which is a string containing JSON.
+    We parse the `raw_data` in order to extract image resources and rewrite links.
 
-        Perseus questions have already been formatted to the
-        .perseus format and can therefore be created with just
-        raw data (no need to parse the data)
-
-        Attributes:
-            id (str): question's unique id
-            raw_data (str): pre-formatted perseus question
-            images ({key:str, ...}): a dict mapping image string to replace to path to image
+    Attributes:
+        id (str): question's unique id
+        raw_data (str): pre-formatted perseus question
+        images (dict): maps image string to replace to path to image `{key: str, ...}`
     """
 
     def __init__(self, id, raw_data, source_url=None, **kwargs):
@@ -227,71 +235,121 @@ class PerseusQuestion(BaseQuestion):
             assert self.question == "", "Assumption Failed: Perseus question should not have a question"
             assert self.question_type == exercises.PERSEUS_QUESTION, "Assumption Failed: Question should be perseus type"
             assert self.answers == [], "Assumption Failed: Answer list should be empty for perseus question"
-            assert self.hints == [], "Assumption Failed: Hint list should be empty for perseus question"
+            assert self.hints == [], "Assumption Failed: Hints list should be empty for perseus question"
             return super(PerseusQuestion, self).validate()
         except AssertionError as ae:
             raise InvalidQuestionException("Invalid question: {0}".format(self.__dict__))
 
     def process_question(self):
-        """ process_question: Parse data that needs to have image strings processed
-            Args: None
-            Returns: list of all downloaded files
+        """
+        Parse specific fields in `self.raw_data` that needs to have image strings
+        processed: repalced by references to `CONTENTSTORAGE` + added as files.
+        Returns: list of all files needed to render this question.
         """
         image_files = []
-        image_data = json.loads(self.raw_data)
+        question_data = json.loads(self.raw_data)
 
         # process urls for widgets
-        self._recursive_url_find(image_data, image_files)
+        self._recursive_url_find(question_data, image_files)
 
         # Process question
-        if 'question' in image_data and 'images' in image_data['question']:
-            image_data['question']['images'], qfiles = self.process_image_field(image_data['question'])
+        if 'question' in question_data and 'images' in question_data['question']:
+            question_data['question']['images'], qfiles = self.process_image_field(question_data['question'])
             image_files += qfiles
 
         # Process hints
-        if 'hints' in image_data:
-            for hint in image_data['hints']:
+        if 'hints' in question_data:
+            for hint in question_data['hints']:
                 if 'images' in hint:
                     hint['images'], hfiles = self.process_image_field(hint)
                     image_files += hfiles
 
         # Process answers
-        if 'answers' in image_data:
-            for answer in image_data['answers']:
+        if 'answers' in question_data:
+            for answer in question_data['answers']:
                 if 'images' in answer:
                     answer['images'], afiles = self.process_image_field(answer)
                     image_files += afiles
 
         # Process raw data
-        self.raw_data = json.dumps(image_data, ensure_ascii=False)
-        self.raw_data, data_files = super(PerseusQuestion, self).set_images(self.raw_data)
+        self.raw_data = json.dumps(question_data, ensure_ascii=False)
+        # Assume no need for special HTML processing for Persues questions
+        # This avoids probelms with questions that contain < and > inequalities
+        # in formulas that get erroneously parsed as HTML tags
+        self.raw_data, data_files = super(PerseusQuestion, self).set_images(self.raw_data, parse_html=False)
 
-        # Return all files
-        self.files += image_files + data_files
+        # Combine all files processed
+        self.files = image_files + data_files
+
+        # Return all filenames
         return [f.filename for f in self.files]
 
-    def process_image_field(self, data):
-        """ process_image_field: Specifically process perseus question image field
-            Args:
-                data (dict): data that contains 'images' field
-            Returns: list of all downloaded files
-        """
-        files = []
 
-        new_data = copy.deepcopy(data['images'])
-        for k, v in data['images'].items():
-            new_key, fs = self.set_image(k)
-            files += fs
-            new_data[new_key] = new_data.pop(k)
-        return new_data, files
+    def process_image_field(self, data):
+        """
+        Process perseus fields like questions and hints, which look like:
+
+        .. code-block:: python
+
+          {
+             "content": "md string including imgs like ![](URL-key) and ![](URL-key2)",
+             "images": {
+                "URL-key":  {"width": 425, "height": 425},
+                "URL-key2": {"width": 425, "height": 425}
+             }
+          }
+
+        Replaces `content` attribute and returns (images_dict, image_files), where
+
+           - `images_dict` is a replacement for the old `images` key
+           - `image_files` is a list image files for the URLs found
+
+        Note it is possible for assesment items to include images links `content`
+        that are not listed under `images`, so code must handle that case too,
+        see https://github.com/learningequality/ricecooker/issues/178 for details.
+        """
+        new_images_dict = copy.deepcopy(data['images'])
+        image_files = []
+
+        # STEP 1. Compile dict of {old_url-->new_url} image URL replacements
+        image_replacements = {}
+
+        # STEP 1A. get all images specified in data['images']
+        for old_url, image_settings in data['images'].items():
+            new_url, new_image_files = self.set_image(old_url)
+            image_files += new_image_files
+            new_images_dict[new_url] = new_images_dict.pop(old_url)
+            image_replacements[old_url] = new_url
+
+        # STEP 1B. look for additional `MARKDOWN_IMAGE_REGEX`-like link in `content` attr.
+        img_link_pat = re.compile(MARKDOWN_IMAGE_REGEX, flags=re.IGNORECASE)
+        img_link_matches = img_link_pat.findall(data['content'])
+        for match in img_link_matches:
+            old_url = match[1]
+            if old_url not in image_replacements.keys():
+                new_url, new_image_files = self.set_image(old_url)
+                image_files += new_image_files
+                image_replacements[old_url] = new_url
+
+        # Performd content replacent for all URLs in image_replacements
+        for old_url, new_url in image_replacements.items():
+            data['content'] = data['content'].replace(old_url, new_url)
+
+        return new_images_dict, image_files
+
 
     def _recursive_url_find(self, item, image_list):
-        """ Utility function specifically for Khan Academy assessment items. Recursively iterates item
-            to find and replace all url image tags.
-            Args:
-                item (dict): KA assessment item
-                image_list (list): list of image file objects
-            Returns: None
+        """
+        Recursively traverses a dictionary-like data structure for Khan Academy
+        assessment items in order to search for image links in `url` data attributes,
+        and if it finds any it adds them to `image_list` and rewrites `url` attribute.
+        Use cases:
+          - `backgroundImage.url` attributes for graphs and images
+
+        Args:
+            item (dict): KA assessment item; will be modified in place
+            image_list (list): image files (File objects) found during the traversal
+        Returns: None
         """
 
         recursive_fn = partial(self._recursive_url_find, image_list=image_list)
@@ -325,7 +383,7 @@ class MultipleSelectQuestion(BaseQuestion):
             question (str): question text
             correct_answers ([str]): list of correct answers
             all_answers ([str]): list of all possible answers
-            hint (str): optional hint on how to answer question
+            hints ([str]): optional hints on how to answer question
             images ({key:str, ...}): a dict mapping image placeholder names to path to image
     """
 
@@ -369,7 +427,7 @@ class SingleSelectQuestion(BaseQuestion):
             question (str): question text
             correct_answer (str): correct answer
             all_answers ([str]): list of all possible answers
-            hint (str): optional hint on how to answer question
+            hints ([str]): optional hints on how to answer question
     """
     def __init__(self, id, question, correct_answer, all_answers, **kwargs):
         # Put answers into standard format
@@ -396,7 +454,7 @@ class SingleSelectQuestion(BaseQuestion):
                 correct_answers += 1 if a['correct'] else 0
             assert correct_answers == 1, "Assumption Failed: Single selection question should have only one correct answer"
             for h in self.hints:
-                assert isinstance(h, str), "Assumption Failed: Hint in hint list is not a string"
+                assert isinstance(h, str), "Assumption Failed: Hint in hints list is not a string"
             return super(SingleSelectQuestion, self).validate()
         except AssertionError as ae:
             raise InvalidQuestionException("Invalid question: {0}".format(self.__dict__))
@@ -412,7 +470,7 @@ class InputQuestion(BaseQuestion):
             id (str): question's unique id
             question (str): question text
             answers ([{'answer':str, 'hint':str}]): answers to question
-            hint (str): optional hint on how to answer question
+            hints ([str]): optional hints on how to answer question
             images ({key:str, ...}): a dict mapping image placeholder names to path to image
     """
     def __init__(self, id, question, answers, **kwargs):
@@ -437,7 +495,7 @@ class InputQuestion(BaseQuestion):
                 except ValueError:
                     assert False, "Assumption Failed: Answer {} must be numeric".format(a['answer'])
             for h in self.hints:
-                assert isinstance(h, str), "Assumption Failed: Hint in hint list is not a string"
+                assert isinstance(h, str), "Assumption Failed: Hint in hints list is not a string"
             return super(InputQuestion, self).validate()
         except AssertionError as ae:
             raise InvalidQuestionException("Invalid question: {0}".format(self.__dict__))
